module Grime
  class ApiController < ApplicationController

    before_action only: [:show, :update, :destroy] do
      @object = scope.find(params[:id])
    end

    rescue_from 'ActiveRecord::RecordInvalid' do |e|
      render json: { errors: e.record.errors }, status: :unprocessable_entity
    end

    rescue_from 'ActionController::UnpermittedParameters' do |e|
      render json: { error: 'unpermitted_parameters', message: e.message }, status: :bad_request
    end

    rescue_from 'ActionController::ParameterMissing' do |e|
      render json: { error: 'missing_parameters', message: e.message }, status: :bad_request
    end

    rescue_from 'ActiveRecord::RecordNotFound' do |e|
      render json: { error: 'not_found' }, status: :not_found
    end

    def index
      if filter_params_present? && scope.respond_to?(:where)
        objects = scope.where(filter_params)
      else
        objects = scope.all
      end
      render json: objects
    end

    def show
      render json: @object
    end

    def create
      @object = scope.create!(object_params_for_create)
      render json: @object
    end

    def update
      @object.update!(object_params_for_update)
      render json: @object
    end

    def destroy
      render json: @object.destroy!
    end

    protected

    def scope
      object_class
    end

    def params_key
      object_class.name.underscore
    end

    def object_class
      controller_name.chomp('Controller').singularize.classify.constantize
    end

    def filter_params_present?
      params[:filter] && params[:filter].is_a?(Hash)
    end

    def filter_params
      logger.warn("WARNING: allowing all filter parameters in #{controller_name}. This is probably not what you want.\nOverride filter_params to fix this.")
      params[:filter].permit!
    end

    def object_params
      logger.warn("WARNING: allowing all parameters in #{controller_name}. This is probably not what you want.\nOverride object_params to fix this.")
      params.require(params_key).permit!
    end

    # NOTE: override these methods if you want to have different
    #       parameters for create vs. update
    #
    def object_params_for_create
      object_params
    end

    def object_params_for_update
      object_params
    end

  end
end
